/*****************************************************************************
 * Program_OTP.c
 *****************************************************************************/

#include "adi_initialize.h"
#include <sys/platform.h>
#include <stdio.h>
#if   defined(__ADSPBF70x__) && __ADSPBF70x__==1
#include <bfrom.h>
#elif defined(__ADSPSC5xx__) && __ADSPSC5xx__==1
#include <cdefSC58x_rom.h>
#include <cdefSC58x_rom_otpLayout.h>
#endif

bool validate_pgm(OTPCMD, uint32_t[], uint8_t);

/* macro to help call the validate function using the otpcmd id */
#define VALIDATE(otpkey) validate_pgm(otpcmd_##otpkey,*key,ROM_OTP_SZ_##otpkey)

/***************************************************
 *
 *  Functions to program the various fields using
 *  keys generated from the keygen project
 *
 ***************************************************/

/*
 * program_public_key
 * program the authentication key using the OTP API
 *
 * applicable for all types unless ADI_ROM_OTP_BOOT_INFO.bcfg.decryptOnlyEn is set to 1
 *
 * secure_boot_public_key.h is generated from the keygen project
 */
bool program_authentication_key(void) {

  int i;
  #include "secure_boot_public_key.h"

  /* because the included array is a byte array, we need to cast it */
  uint32_t (*key)[ROM_OTP_SZ_public_key0] = &secure_boot_public_key;

  /* print out key */
  printf("Programming  auth key:\n");
  printf("\tQx: ");
  for(i=0; i<28; i++)
    printf("%02x ", secure_boot_public_key[i]);
  printf("\n");
  printf("\tQy: ");
  for(i=28; i<2*28; i++)
    printf("%02x ", secure_boot_public_key[i]);
  printf("\n\n");

  /* ensure all other values are 0, so no accidental write occurs */
  otp_data data = {0};
  data.public_key0 = key;

  /* call the api */
  if(!adi_rom_otp_pgm(&data)) { return false; }

  /* verify the data was written */
  return VALIDATE(public_key0);

}

/* program_encryption_key
 *
 * program the encryption/decryption key using the OTP API
 * This is applicable for the Blx image type
 *
 * encrypt_key.h is generated from the keygen project
 *
 */
bool program_encryption_key(void) {

  int i;
  #include "encrypt_key.h"

  /* because the included array is a byte array, we need to cast it */
  uint32_t (*key)[ROM_OTP_SZ_pvt_128key0] = &encrypt_key;

  /* print out key */
  printf("Programming encrypt key: ");
  for(i=0; i<encrypt_key_bytes_sz; i++)
    printf("%02x ", encrypt_key[i]);
  printf("\n\n");

  /* ensure all other values are 0, so no accidental write occurs */
  otp_data data = {0};
  data.pvt_128key0 = key;

  /* call the api */
  if(!adi_rom_otp_pgm(&data)) { return false; }

  /* verify the data was written */
  return VALIDATE(pvt_128key0);

}

/* program_confidentiality_key
 *
 * program the wrapper key using the OTP API
 * This is applicable for the BLw image type
 *
 * wrapper_key.h is generated by keygen project
 *
 */
bool program_confidentiality_key(void) {

  int i;
  #include "wrapper_key.h"

  /* because the included array is a byte array, we need to cast it */
  uint32_t (*key)[ROM_OTP_SZ_pvt_128key0] = &wrapper_key;

  /* print out key */
  printf("Programming wrapper key: ");
  for(i=0; i<wrapper_key_bytes_sz; i++)
    printf("%02x ", wrapper_key[i]);
  printf("\n\n");

    /* ensure all other values are 0, so no accidental write occurs */
  otp_data data = {0};
  data.pvt_128key0 = key;

  /* call the api */
  if(!adi_rom_otp_pgm(&data)) { return false; }

  /* verify the data was written */
  return VALIDATE(pvt_128key0);

}

/* program_emulation_key
 *
 * use this to program the emuluation (jtag) key using the OTP API
 * this is required for secure debug access. 0 is also a valid debug key
 * secure emulation can be completely disabled by setting the key to all 1s.
 *
 * jtag_key.h is generated by the keygen project
 *
 * */
bool program_emulation_key(void) {

  int i;
  #include "jtag_key.h"

  /* because the included array is a byte array, we need to cast it */
  uint32_t (*key)[ROM_OTP_SZ_secure_emu_key] = &jtag_key;

  /* print out key */
  printf("Programming JTAG key: ");
  for(i=0; i<jtag_key_bytes_sz; i++)
	  printf("%02x ", jtag_key[i]);
  printf("\n\n");


  /* ensure all other values are 0, so no accidental write occurs */
  otp_data data = {0};
  data.secure_emu_key = key;

  /* call the api */
  if(!adi_rom_otp_pgm(&data)) { return false; }

  /* verify the data was written */
  return VALIDATE(secure_emu_key);

}

/* lock_part
 *
 * Program the lock bit in OTP, after this the part is secured
 *
 */
bool lock_part(void) {

  printf("locking part!\n\n");

  if(!adi_rom_otp_lock()) { return false; }
  return true;
}


/*************************************************************************
 *
 * Functions to prepare the part for the different Image Type scenarios
 *
 *************************************************************************/
bool setupBlp(void)  {
	if(!program_authentication_key()) { printf("failed to program authentication key\n"); return false; }
	if(!program_emulation_key())      { printf("failed to program the emulation key\n"); return 3; }
	if(!lock_part())                  { printf("failed to lock the part\n"); return false; }
	return true;
}

bool setupBlx(void) {
	if(!program_authentication_key()) { printf("failed to program authentication key\n"); return false; }
	if(!program_encryption_key()) 	  { printf("failed to program encryption key\n"); return false; }
	if(!program_emulation_key())      { printf("failed to program the emulation key\n"); return false; }
	if(!lock_part())                  { printf("failed to lock the part\n"); return false; }
	return true;
}

bool setupBlw(void) {
	if(!program_authentication_key())  { printf("failed to program authentication key\n"); return false; }
	if(!program_confidentiality_key()) { printf("failed to program wrapper key\n"); return false; }
	if(!program_emulation_key()) 	   { printf("failed to program the emulation key\n"); return false; }
	if(!lock_part())                   { printf("failed to lock the part\n"); return false; }
	return true;
}


uint32_t buf[90]; /* temp buffer used for reading */


/*
 * example for programming a field
 */
bool program_gp1() {

  uint32_t gp1[ROM_OTP_SZ_gp1]={0x3F,0x3};
  otp_data data = {0};
  data.gp1 = &gp1;
  if(!adi_rom_otp_pgm(&data)) { return false; }
  return validate_pgm(otpcmd_gp1,gp1,ROM_OTP_SZ_gp1);

}


/* usePublicKey1
 *
 * example of invalidating a key, and using the next one
 * All keys and many field in OTP have a complimentary Inv field
 * For example ADI_ROM_OTP_BOOT_INFO.bcfg.decryptOnlyEn, and ADI_ROM_OTP_BOOT_INFO.bcfg.decryptOnlyEnInv
 * When the Inv field is set the associated field is invalidated.
 *
 * For the case of keys, when a key is invalidated the next enumerated key
 * is used. In this case if we set  ADI_ROM_OTP_BOOT_INFO.bcfg.pubkey0Inv the public key0 is
 * invalidated, and public key1 is used
 *
 *
 */
bool usePublicKey1(uint32_t newKey[]) {

  otp_data data = {0};
  ADI_ROM_OTP_BOOT_INFO bootInfo = {0};
  bootInfo.bcfg.privkey0Inv = 1;

  data.boot_info = &bootInfo;
  data.public_key0 = &newKey;

  if(!adi_rom_otp_pgm(&data)) { return false; }

  return true;

}


/*!
 * read_public_key0
 *
 * read a key into buf
 *
 */
bool read_public_key0() {
    return adi_rom_otp_get(otpcmd_public_key0,buf);
}


/*!
 * validate_pgm
 * validates that the field given by cmd matched the expected value
 * global buffer buf required
 *
 * cmd OTPCMD  associated with the data to be validated
 * sizeInWords  the size of the data in 32bit words
 *
 *
 */
bool validate_pgm(OTPCMD cmd, uint32_t expected[], uint8_t sizeInWords) {

  if(!adi_rom_otp_get(cmd,buf)) {return false;}

  int i = 0;
  for(i = 0; i<sizeInWords; i++) {
    if ( buf[i] != expected[i] ) { return false; }
  }

  return true;
}





int main(void)
{
	uint8_t *pk;
	int i;

	adi_initComponents();

	/****************************************************************************
	 *
	 *  The following three functions will program the applicable OTP fields
	 *  for the associated image type. Only one can be used on a single part
	 *
	 *  All the keys used are generated from the keygen project. The keygen
	 *  project will only generate keys once, and used to program the loader
	 *  stream and program the OTP fields
	 *
	 *****************************************************************************/

	//setupBlp();
	//setupBlw();
	//setupBlx();

	read_public_key0();
    /* print out key */
	pk = (uint8_t *)buf;
	printf("Reading  auth key:\n");
	printf("\tQx: ");
	for(i=0; i<28; i++)
	  printf("%02x ", pk[i]);
	printf("\n");
	printf("\tQy: ");
	for(i=28; i<2*28; i++)
	  printf("%02x ", pk[i]);
	printf("\n\n");


	lock_part();


	return 0;
}

